/**********************************************
 SOURCE : myBrowse.prg
 SYSTEM : generic
 AUTHOR : Phil Ide
 PURPOSE: Mybrowse CLASS (XbpBrowse() wrapper)

 Copyright (c) Phil Ide 2001, All Rights Reserved

 NOTES:
 This class is called in the same way as XbpBrowse but adds the following enhancements:
   ::cAlias ivar holds a reference to the workarea alias the browse is associated with.
            The default is the current alias at the time the object is created.
   ::cChild ivar holds a reference to a child-browse object to be refreshed each time
            this browse becomes stable.
   ::whileClause ivar holds a code-block to define scoping.
            The default codeblock returns TRUE.
   ::setup() method is called automatically when the object is created.
            It creates default codeblocks for the following XbpBrowse navigation slots:
      ::cursorMode
      ::softTrack
      ::setFontCompoundName()
      ::hScroll
      ::itemSelected
      ::skipBlock
      ::goTopBlock
      ::goBottomBlock
      ::phyPosBlock
      ::posBlock
      ::goPosBlock
      ::lastPosBlock
      ::firstPosBlock

  Each ivar can of course be altered after initialising the object.
  Note that the navigation slots utilise the ::cAlias ivar - an example: 
        ::goTopBlock    := {|| (::cAlias)->(DbGoTop()) }
        
  In short, this class allows a GUI browse to be created and used with mimimal
  effort (i.e. create the object, add the columns and go...)  

**********************************************/

#include "Gra.ch"
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"
#include "Font.ch"

#pragma Library("xppui2.lib")

#define CLASS_NAME myPGSQLBrowse


class CLASS_NAME from xbpbrowse
   exported:
      var whileclause
      var cAlias
      var cChild
      var oPG

      method init
      method setup
      method dbskipper
      method create
endclass

method CLASS_NAME:init( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::xbpbrowse:init( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::whileclause := {|| TRUE }
   ::cAlias := Alias()
//   ::setup()
   return (self)

method CLASS_NAME:create( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::xbpbrowse:create()
   return (self)

method CLASS_NAME:setup(oPGResult)
   ::oPG := oPGResult

   ::cursorMode := XBPBRW_CURSOR_ROW
   ::softTrack := TRUE
   ::setFontCompoundName(FONT_DEFFIXED_SMALL)
   ::hScroll := TRUE
   ::itemSelected := {|| NIL }

   // Navigation code blocks for the browser
   ::skipBlock     := {|n| (::DbSkipper(n)) }
   ::goTopBlock    := {| | (::oPG:GoTop())      }
   ::goBottomBlock := {| | (::oPG:GoBottom())   }

   ::phyPosBlock   := {| | (::oPG:recNo())        }

   // Navigation code blocks for the vertical scroll bar
   ::posBlock      := {| | (::oPG:Position())    }
   ::goPosBlock    := {|n| (::oPG:GoPosition(n)) }
   ::lastPosBlock  := {| | 100             }
   ::firstPosBlock := {| | 0               }

RETURN (self)

method CLASS_NAME:dbSkipper( n )
   local nDir
   local nDone := 0

   default n to 0

   nDir := iif( n > 0, 1, -1 )

   while n <> 0 .and. iif( nDir > 0, !::oPG:eof(), !::oPG:bof() )
      ::oPG:Skip(nDir)
      if ::oPG:eof()
         exit
      elseif ::oPG:bof()
         exit
      endif
      if !Eval(::whileClause)
         ::oPG:Skip(-nDir)
         if !Eval(::whileClause) // oops, no recs available
            ::oPG:GoBottom()
            ::oPG:Skip()
         endif
         exit
      endif
      n     -= nDir
      nDone += nDir
   enddo
   if valtype(::cChild) == "O"
      ::cChild:goTop()
      ::cChild:refreshAll()
   endif
   return (nDone)

